import { Observable, of } from 'rxjs';
import { TodoService } from '@core/ports/todo.service';
import { Todo, Todos } from '@core/models/todo';
import { TodoBuilder } from '@core/models/builder/todo.builder';

export class InMemoryTodoService extends TodoService {
  private todos: Todos = [];

  withTodos(todos: Todos): this {
    this.todos = todos;
    return this;
  }

  override getAll() {
    return of(this.todos);
  }

  override add(todoName: string): Observable<Todo> {
    const todo: Todo = new TodoBuilder()
      .withId(this.todos.length + 1)
      .withName(todoName)
      .uncomplete()
      .build();
    this.todos = [...this.todos, todo];
    return of(todo);
  }

  override toggleCompleted(todoId: number): Observable<Todo> {
    const todo = this.todos.find((todo) => todo.id === todoId)!;
    todo.completed = !todo.completed;
    return of(todo);
  }

  override remove(todoId: number): Observable<void> {
    this.todos = this.todos.filter((todo) => todo.id !== todoId);
    return of(void 0);
  }
}
