import { Component, inject } from '@angular/core';
import { TodoService } from '@core/ports/todo.service';
import { toSignal } from '@angular/core/rxjs-interop';
import { BehaviorSubject, switchMap, tap } from 'rxjs';
import { FormsModule } from '@angular/forms';
import { Todo } from '@core/models/todo';
import { AddTodoFormComponent } from '@features/todo-list/components/add-todo-form.component';
import { TodoListItemComponent } from '@features/todo-list/components/todo-list-item';

@Component({
  selector: 'app-todo-list',
  imports: [FormsModule, AddTodoFormComponent, TodoListItemComponent, TodoListItemComponent],
  templateUrl: './todo-list.component.html',
  styleUrl: './todo-list.component.scss',
})
export default class TodoListComponent {
  private reload$$ = new BehaviorSubject<void>(void 0);
  private readonly todoService = inject(TodoService);
  todos = toSignal(
    this.reload$$.pipe(
      switchMap(() => this.todoService.getAll()),
      tap((t) => console.log('Tâches mises à jour:', t))
    )
  );

  addTodo(todoName: string) {
    if (!todoName) return;
    this.todoService
      .add(todoName)
      .pipe(tap(() => this.reload$$.next()))
      .subscribe();
  }

  toggleCompleted(todo: Todo) {
    this.todoService
      .toggleCompleted(todo.id)
      .pipe(tap(() => this.reload$$.next()))
      .subscribe();
  }

  remove(todo: Todo) {
    this.todoService
      .remove(todo.id)
      .pipe(tap(() => this.reload$$.next()))
      .subscribe();
  }
}
